/*
 * Copyright 2002-2005 webapp2.net
 *
 * 본 파일은 the Apache License, Version 2.0를 따릅니다:
 *
 * 	http://www.apache.org/licenses/LICENSE-2.0
 *
 */
package net.webapp2.member.dao;

import java.util.Date;

import net.webapp2.member.domain.Member;
import net.webapp2.util.DBUtil;

import org.springframework.test.AbstractTransactionalSpringContextTests;

/**
 * @author Park, chanwook
 * @author Beak, keesun
 *
 */
public class MemberDaoTest extends AbstractTransactionalSpringContextTests {

	private MemberDao memberDao;

	public void setMemberDao(MemberDao memberDao) {
		this.memberDao = memberDao;
	}

	public void testInsert() {
		String id = "whiteship2000";
		String name = "KeeSun";
		String password = "12323";
		String email = "whiteship2000@gmai.com";

		Member member = new Member();

		assertTrue(DBUtil.checkId(id));
		member.setId(id);

		assertTrue(DBUtil.checkPassword(password));
		member.setPassword(password);

		assertTrue(DBUtil.checkEmail(email));
		member.setEmail(email);

		assertTrue(DBUtil.checkName(name));
		member.setName(name);
		member.setAlias("");
		member.setJoined(new Date());

		memberDao.insert(member);
	}


	public void testLogin(){
		Member member = memberDao.login("whiteship", "12323");

		assertNotNull(member.getId());
	}

	public void testUpdate(){

		//사용자 입력값
		//업데이트 하고자 하는 member의 id, password
		String id="whiteship";

		Member member = memberDao.load(id);
		assertNotNull(member);

		//member의 객체를 받아와서 member의 정보를 수정하고 update 함
		//사용자 정보 수정
		String password = "23141";
		String alias = "변경된 별명";
		String blog = "변경된 블로그";
		String email = "변경될  이메일";

		assertTrue(DBUtil.checkPassword(password));
		member.setPassword(password);
		member.setAlias(alias);
		member.setBlog(blog);
		assertTrue(DBUtil.checkEmail(email));
		member.setEmail(email);

		memberDao.update(member);

		//update가 정확히 됐는지 확인
		Member mem = memberDao.login(id, password);
		assertEquals(member.getPassword(), mem.getPassword());
	}

	public void testDelete(){
		//login 되었다 가정
		String id = "whiteship";
		memberDao.delete(id);

		Member member = memberDao.load(id);
		assertNull(member);
	}

	public void testLoad(){
		String id = "whiteship";

		Member member = memberDao.load(id);
		assertNotNull(member);
	}


	public void testFind(){
		//맞지 않은 property 값을 주고 검색하는 것 잡아내기

	}

	@Override
	protected String[] getConfigLocations() {
		return new String[] { "file:conf/applicationContext-member.xml",
				"file:conf/daoContext.xml",
				"file:conf/daoContext-member.xml",
				"file:conf/sql-map-config.xml" };
	}

}
