/*
 * Copyright 2002-2005 webapp2.net
 *
 * 본 파일은 the Apache License, Version 2.0를 따릅니다:
 *
 * 	http://www.apache.org/licenses/LICENSE-2.0
 *
 */
package net.webapp2.member.service;

import java.util.Date;
import java.util.List;

import net.webapp2.member.domain.Member;
import net.webapp2.member.domain.Role;
import net.webapp2.util.DBUtil;

import org.springframework.test.AbstractDependencyInjectionSpringContextTests;

/**
 * @author Park, chanwook
 * @author Beak, keesun
 *
 */
public class MemberServiceTest extends AbstractDependencyInjectionSpringContextTests {

	private MemberService memberService;

	/**
	 * 회원 가입
	 */
	public void testJoin() {

		// 사용자 입력값
		String id = "number";
		String password = "aaaasas123";
		String name = "aa";
		String email = "ahnyounghoe@empal.com";

		Member member = new Member();

		assertTrue(DBUtil.checkId(id));
		member.setId(id);

		assertTrue(DBUtil.checkPassword(password));
		member.setPassword(password);

		assertTrue(DBUtil.checkEmail(email));
		member.setEmail(email);

		assertTrue(DBUtil.checkName(name));
		member.setName(name);
		member.setAlias("");
		member.setJoined(new Date());

		memberService.join(member);

		assertNotNull(member.getJoined());
	}

	/**
	 * login
	 */
	public void testLogin() {

		// 사용자 입력값
		//paswword는 체크하지 않았음
		String id = "whiteship";
		String password = "12323";

		Member member = memberService.login(id, password);

		assertNotNull(member.getId());
	}

	/**
	 * 회원 정보 수정
	 */
	public void testUpdate(){
		/** Role은 사용자가 수정하는 것이 아니라 판단하여 sqlmap 처리 하지 않음
		 *  id는 최초에 주는 값이기 때문에 수정 불가능
		 *  joined는 최초 가입일이기 때문에 수정 불가능
		 */

		//사용자 입력값
		//업데이트 하고자 하는 member의 id를 이용해서 멤버 정보 불러옴
		String id="www";

		Member member = memberService.load(id);
		assertNotNull(member);

		//member의 객체를 받아와서 member의 정보를 수정하고 update 함
		//사용자 정보 수정
		String password = "23141";
		String alias = "변경된 별명";
		String blog = "변경된 블로그";
		String email = "변경될  이메일";

		assertTrue(DBUtil.checkPassword(password));
		member.setPassword(password);
		member.setAlias(alias);
		member.setBlog(blog);
		assertTrue(DBUtil.checkEmail(email));
		member.setEmail(email);

		memberService.update(member);
	}


	public void testDelete(){
		//login 되었다 가정
		String id = "number";
		memberService.delete(id);

		Member member = memberService.load(id);
		assertNull(member);
	}

	/**
	 * 특정 회원 검색
	 *
	 */
	public void testLoad(){
		String id = "whiteship";

		Member member = memberService.load(id);
		assertNotNull(member);
	}

	/**
	 * 전체 회원 list
	 *
	 */
	public void testList(){
		List<Member> list = memberService.list();
		assertNotNull(list);
		System.out.println(list.get(0).getName());
	}

	/**
	 * 회원 찾기
	 * id, name, email, alias 네 가지 찾기 구현.
	 * 네 가지 모두 같은 메소드인 find메소드 사용.
	 * like 구문 사용. 일부라도 매치되는 내용을 리턴
	 *
	 */
	public void testFind(){

		//id로 회원찾기
		List<Member> member = memberService.find("id", "w");
		assertNotNull(member);
		System.out.println(member.get(0).getId());

		//name으로 회원찾기
//		Member member = memberService.find("name","백기선");
//		assertNotNull(member);
//		System.out.println(member.get(0).getName());
//		System.out.println(member.get(1).getName());

		//email로 회원찾기
//		Member member = memberService.find("email", "whiteship2000@gmail.com");
//		assertNotNull(member);
//		System.out.println(member.get(0).getEmail());
//		System.out.println(member.get(1).getemail());

		//alias로 회원찾기
//		Member member = memberService.find("alias", "별명");
//		assertNotNull(member);
//		System.out.println(member.get(0).getAlias());
//		System.out.println(member.get(1).getAlias());
	}


	/**
	 * Role에 관련된 기능.
	 * provideRole() : 특정 회원에 역할 부여
	 * loadRoleInfo() : 특정 role을 가진 회원을 검색하거나, 한 회원이 가진 모든 role 검색
	 *
	 */
	public void testWithRole(){
		//특정 회원에게 role 역할 추가
		String role_name = "관리자";
		String member_id = "whiteship";
		memberService.provideRole(role_name, member_id);

		//role 정보 검색 - 특정 role_name을 가진 Role 객체 반환
//		String property = "role_name";
//		String value = "관리자";
//		List<Role> role = memberService.loadRoleInfo(property, value);
//		System.out.println(role.get(0).getFk_member_id());

		//role 정보 검색 - 특정 회원의 role을 Role 객체로 반환
		String property = "fk_member_id";
		String value = "whiteship";
		List<Role> role = memberService.loadRoleInfo(property, value);
		System.out.println(role.get(0).getRole_name());

	}

	@Override
	protected String[] getConfigLocations() {
		return new String[] { "file:conf/applicationContext-member.xml",
				"file:conf/daoContext.xml",
				"file:conf/daoContext-member.xml",
				"file:conf/sql-map-config.xml" };
	}

	public void setMemberService(MemberService memberService) {
		this.memberService = memberService;
	}

}
