package net.webapp2.article.domain;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang.builder.ToStringBuilder;

/**
 * 기술문서 정보
 * @author younghoe
 *
 */
public class Article {

	/**
	 * 설명 혹은 요약
	 */
	private String description;

	/**
	 * 정수 형태의 관리번호
	 */
	private Integer id;


	/**
	 * 영한 구분
	 */
	private Language language;

	/**
	 * 기술문서 경로 URL
	 */
	private String link;

	/**
	 * 게시판에 걸린 시간
	 */
	private Date postedTime;

	/**
	 * 분류목록
	 */
	private List<Category> tagList = new ArrayList<Category>();

	/**
	 * 제목
	 */
	private String title;

	/**
	 * 최종 수정일시
	 */
	private Date lastModified;


	/**
	 * 관리번호를 부여한 생성자
	 * @param id 관리번호
	 */
	public Article(Integer id) {
		this.id = id;
	}

	public Article() {
	}

	public void setTitle(String title) {
		this.title = title;
	}


	public void setDescription(String description) {
		this.description = description;
	}

	public void setLanguage(Language lang) {
		this.language = lang;
	}

	public String getTitle() {
		return this.title;
	}

	public Integer getId() {
		return id;
	}

	public void setId(Integer id) {
		this.id = id;
	}

	public Date getPostedTime() {
		return postedTime;
	}

	public void setPostedTime(Date postedTime) {
		this.postedTime = postedTime;
	}

	public Date getLastModified() {
		return lastModified;
	}

	public void setLastModified(Date lastModified) {
		this.lastModified = lastModified;
	}

	public String getDescription() {
		return description;
	}

	public Language getLanguage() {
		return language;
	}

	/**
	 * 특정 주제어로 꼬리표 부여
	 * @param category 꼬리표가 되는 주제어
	 */
	public void tag(Category category) {
		tagList.add(category);
	}

	/**
	 * 정 꼬리표가 붙었는지 여부 확인
	 * @param category 꼬리표
	 * @return
	 */
	public boolean isTagged(Category category) {
		return tagList.contains(category);
	}

	public String getLink() {
		return link;
	}

	public void setLink(String link) {
		this.link = link;
	}

	public List<Category> getTagList() {
		return tagList;
	}

	public void setTagList(List<Category> tagList) {
		this.tagList = tagList;
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return new ToStringBuilder(this).append("postedTime", this.postedTime)
				.append("description", this.description).append("id", this.id)
				.append("title", this.title).append("lastModified",
						this.lastModified).append("link", this.link).append(
						"language", this.language).append("tagList",
						this.tagList).toString();
	}

}