package net.webapp2.article.dao.ibatis;

import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.sql.DataSource;

import net.webapp2.article.dao.ArticleDao;
import net.webapp2.article.domain.Article;
import net.webapp2.article.domain.Category;
import net.webapp2.article.domain.Language;
import net.webapp2.util.date.DateUtils;
import net.webapp2.util.jdbc.SimpleSqlExecutor;
import net.webapp2.util.test.AssertUtils;
import net.webapp2.util.test.TestBase;

import org.springframework.dao.DataIntegrityViolationException;
import org.testng.annotations.Configuration;
import org.testng.annotations.ExpectedExceptions;
import org.testng.annotations.Test;

public class ArticleDaoSupportTest extends TestBase {

	ArticleDao articleDao;

	@Test
	public void insert_주제어_입력() throws MalformedURLException{

		Article article = 아티클입력();
		List<Category> categoryList = article.getTagList();
		int i = 1;

		for (Category category : categoryList) {
			category.setId(i++);
			articleDao.insert(category);
		}
	}

	@Test
	public void insert_아티클_입력() throws MalformedURLException{

		Article article = 아티클입력();
		articleDao.insert(article);
	}

	@Test
	public boolean select_카테고리_존재유무(String categoryName){


		return false;
	}

	@Test(groups="예외상황")
	@ExpectedExceptions(DataIntegrityViolationException.class)
	public void insert_not_null_필드_없이_아티클_입력_시도() throws MalformedURLException{

		Article article = 아티클입력();
		// article.setLanguage(null);
		// article.setId(null);
		// article.setPostedTime(null);
		article.setTitle(null);

		ArticleDao articleDao = DAO획득();
		아티클_및_카테고리입력(article, articleDao);

	}

	@Test(groups="예외상황")
	@ExpectedExceptions(MalformedURLException.class)
	public void insert_잘못된_URL_을_갖는_아티클_입력_시도() throws MalformedURLException{

		Article article = 아티클입력();
		article.setLink(null);

		ArticleDao articleDao = DAO획득();
		아티클_및_카테고리입력(article, articleDao);

	}

	/**
	 * 1. getTagList statement에서 namespace를 붙이지 않으니
	 * UncategorizedSQLException 발생
	 * @throws MalformedURLException
	 */
	@Test
	public void 분류_및_조회() throws MalformedURLException{

		Article article = 아티클입력();

		ArticleDao articleDao = DAO획득();

		아티클_및_카테고리입력(article, articleDao);

		Article fromDb = articleDao.getArticle(article.getId());

		AssertUtils.equals(article.getId(), fromDb.getId());

		for (Category tag : fromDb.getTagList()) {
			AssertUtils.isTrue(article.getTagList().contains(tag));
		}

		AssertUtils.equals(article.getDescription(), fromDb.getDescription());
		AssertUtils.equals(article.getLanguage(), fromDb.getLanguage());
		AssertUtils.equals(article.getLink(), fromDb.getLink());
		AssertUtils.equals(article.getPostedTime(), fromDb.getPostedTime());
		AssertUtils.equals(article.getTitle(), fromDb.getTitle());
	}

	private void 아티클_및_카테고리입력(Article article, ArticleDao articleDao) {
		articleDao.insert(article);

		int i=0;
		for (Category category : article.getTagList()) {
			category.setId(i++);
			articleDao.insert(category);
		}
	}

//	@Test
//	public void save_수정_후_저장() throws MalformedURLException{
//
//		ArticleDao articleDao = (ArticleDao) getBean("articleDao");
//		Article entry = getArticle();
//		articleDao.insert(entry);
//
//		Article article = articleDao.get(entry.getId());
//		article.setLastModified(new Date());
//		article.setDescription("마음에 들어요.");
//		article.tag("iBatis");
//
//		articleDao.save(article);
//
//	}


//	@Test(dependsOnMethods = { "getArticle" })
//	public void updateArticle() throws MalformedURLException {
//
//		ArticleDao articleDao = (ArticleDao) getBean("articleDao");
//		Article article = articleDao.get(testArticleId);
//
//		String updatedDescription = "좋은 글이삼";
//		String updatedLink = "http://guide.webapp2.net";
//		String updatedTitle = "Spring and Summer";
//		String updatedLanguage = "kr";
//		Date updatedPostedTime = DateUtils.getDate(2005, 11, 8);
//		Date lastUpdatedTime = new Date();
//
//		AssertJUnit.assertNotSame(updatedDescription, article.getDescription());
//		AssertJUnit.assertNotSame(updatedLink, article.getLink());
//		AssertJUnit.assertNotSame(updatedTitle, article.getTitle());
//		AssertJUnit.assertNotSame(updatedLanguage, article.getLanguage());
//		AssertJUnit.assertNotSame(updatedPostedTime, article.getPostedTime());
//		AssertJUnit.assertNotSame(lastUpdatedTime, article.getLastModified());
//
//		// 수정
//		Article updatedArticle = new Article();
//		updatedArticle.setId(article.getId());
//		updatedArticle.setDescription(updatedDescription);
//		updatedArticle.setLink(updatedLink);
//		updatedArticle.setTitle(updatedTitle);
//		updatedArticle.setLastModified(lastUpdatedTime);
//		updatedArticle.setPostedTime(updatedPostedTime);
//		updatedArticle.setLanguage(updatedLanguage);
//
//		int row = articleDao.update(updatedArticle);
//		AssertJUnit.assertEquals(1, row);
//
//		updatedArticle = articleDao.get(testArticleId);
//		AssertJUnit.assertEquals(updatedDescription, updatedArticle.getDescription());
//		AssertJUnit.assertEquals(updatedLink, updatedArticle.getLink());
//		AssertJUnit.assertEquals(updatedTitle, updatedArticle.getTitle());
//		AssertJUnit.assertEquals(updatedLanguage, updatedArticle.getLanguage());
//		AssertJUnit.assertEquals(updatedPostedTime, updatedArticle.getPostedTime());
//		AssertJUnit.assertEquals(lastUpdatedTime.getTime(), updatedArticle
//				.getLastModified().getTime());
//
//	}
//
//	@Test(dependsOnMethods = "updateArticle")
//	public void updateArticleWithTags(){
//		ArticleDao articleDao = (ArticleDao) getBean("articleDao");
//		Integer id = articleDao.get(testArticleId).getId();
//		Article articleToBeUpdated = new Article(id);
//		articleToBeUpdated.tag(Category.SPRING);
//		articleToBeUpdated.tag(Category.AOP);
//		articleToBeUpdated.tag(Category.DAO);
//		articleToBeUpdated.tag(Category.WEB);
//
//		int rows = articleDao.update(articleToBeUpdated);
//		System.out.println("변경된 레코드 수: " + rows);
//		Article updatedArticle = articleDao.get(id);
//		System.out.println(updatedArticle.toString());
//
//		AssertJUnit.assertTrue(updatedArticle.isTaggedAs(Category.SPRING));
//		AssertJUnit.assertTrue(updatedArticle.isTaggedAs(Category.AOP));
//		AssertJUnit.assertTrue(updatedArticle.isTaggedAs(Category.DAO));
//		AssertJUnit.assertTrue(updatedArticle.isTaggedAs(Category.WEB));
//
//	}
//
//	@Test(dependsOnMethods = "updateArticleWithTags")
//	public void removeArticle(){
//
//		ArticleDao articleDao = (ArticleDao) getBean("articleDao");
//		Article article = articleDao.get(testArticleId);
//		AssertJUnit.assertNotNull(article);
//
//		int row = articleDao.delete(article.getId());
//		AssertJUnit.assertEquals(1, row);
//
//		article = articleDao.get(testArticleId);
//		AssertJUnit.assertNull(article);
//
//	}
//
//	@Test
//	public void verifyCount(){
//
//		ArticleDao articleDao = (ArticleDao) getBean("articleDao");
//		Integer count = articleDao.count();
//		Integer realCount = articleDao.countAsItIs().get(ArticleDao.ACTUAL_ARTICLE_COUNT);
//		System.out.println("통계치: " + count + ", 실제: " + realCount);
//
//	}
//
//	@Test(dependsOnMethods="verifyCount")
//	public void resetStat(){
//
//		ArticleDao articleDao = (ArticleDao) getBean("articleDao");
//		Integer realCount = 5;		// 실제 레코드 수량
//		Integer issueCount = 10;	// 최종적으로 부여된 관리번호 = total 레코드의 cnt값
//
//		// 데이터 무결성 규칙
//		// Real Count = Issue Count - Del Count
//		Integer delCount = issueCount - realCount;
//		articleDao.resetStat(issueCount, delCount);
//
//		Integer countAfterReset = articleDao.count();
//		AssertJUnit.assertEquals(realCount, countAfterReset);
//
//	}
//
//	@Test(dependsOnMethods = "removeArticle")
//	@Parameters("numberOfArticles")
//	public void allArticleList(int numberOfArticles){
//
//		ArticleDao articleDao = (ArticleDao) getBean("articleDao");
//		List<Article> allArticleList = articleDao.allArticleList();
//
//		assert allArticleList != null;
//
//		AssertJUnit.assertEquals(numberOfArticles, allArticleList.size());
//
//	}

	/////////////////////////////////////////////////////////////////////
	// non-test method 												   //
	/////////////////////////////////////////////////////////////////////



	private Article 아티클입력() throws MalformedURLException {

		Integer id = 1;
		String title = "A Brief Introduction to IoC";
		String link = "http://today.java.net/lpt/a/73";
		String description = "IoC에 대한 간략한 소개글입니다.";
		Date date = DateUtils.getDate(2006, 4, 5);

		List<Category> categoryList = new ArrayList<Category>();

		categoryList.add(new Category("Spring"));
		categoryList.add(new Category("iBatis"));
		categoryList.add(new Category("Webapp2"));

		Article entry = new Article();
		entry.setId(id);
		entry.setTitle(title);
		entry.setLink(link);
		entry.setDescription(description);
		entry.setPostedTime(date);
		entry.setTagList(categoryList);
		entry.setLanguage(Language.kr);

		return entry;
	}

	@Configuration(beforeTestMethod=true)
	private ArticleDao DAO획득() {
		articleDao = (ArticleDao) getBean("articleDao");
		return articleDao;
	}

	@SuppressWarnings("unused")
	@Configuration(beforeTestMethod=true)
	private void 테스트_데이터베이스_초기화() {

		SimpleSqlExecutor executor =  new SimpleSqlExecutor();
		executor.setDataSource((DataSource) getBean("dataSource"));

		// for Total table
		String dropTableStatement = "DROP TABLE ManagedObject IF EXISTS CASCADE";
		executor.execute(dropTableStatement);

		String createTableStatement = "CREATE TABLE ManagedObject (" +
				"type VARCHAR PRIMARY KEY, issue INTEGER, delete INTEGER)";
		executor.execute(createTableStatement);

		String initStatement = "INSERT INTO ManagedObject VALUES('category', 0, 0)";
		executor.execute(initStatement);

		initStatement = "INSERT INTO ManagedObject VALUES('article', 0, 0)";
		executor.execute(initStatement);

		// for Article table
		dropTableStatement = "DROP TABLE Article IF EXISTS CASCADE";
		executor.execute(dropTableStatement);


		createTableStatement = "CREATE TABLE Article ("
				+ "id INTEGER PRIMARY KEY, "
				+ "title VARCHAR(255) NOT NULL, "
				+ "link VARCHAR(255) NOT NULL, "
				+ "language CHAR(2) NOT NULL, "
				+ "postedTime TIMESTAMP NOT NULL, "
				+ "lastModified TIMESTAMP, "
				+ "description VARCHAR"
				+ ")";

		executor.execute(createTableStatement);

		// for Category table
		dropTableStatement = "DROP TABLE Category IF EXISTS CASCADE";
		executor.execute(dropTableStatement);

		createTableStatement = "CREATE TABLE Category (" +
				"id INTEGER PRIMARY KEY, " +
				"name VARCHAR(25) NOT NULL " +
				")";
		executor.execute(createTableStatement);

		// for Tag table
		dropTableStatement = "DROP TABLE Tag IF EXISTS CASCADE";
		executor.execute(dropTableStatement);

		createTableStatement = "CREATE TABLE Tag (" +
				"article_ref INTEGER, " +
				"category_ref INTEGER, " +
				"FOREIGN KEY (article_ref) REFERENCES Article(id)," +
				"FOREIGN KEY (category_ref) REFERENCES Category(id)" +
				")";
		executor.execute(createTableStatement);

	}

}
