package net.webapp2.common.managedobject.dao.ibatis;

import java.util.HashMap;
import java.util.Map;

import net.webapp2.common.managedobject.dao.ManagedObjectDao;

import org.springframework.orm.ibatis.support.SqlMapClientDaoSupport;

public class ManagedObjectDaoSupport extends SqlMapClientDaoSupport implements
ManagedObjectDao{

	private Map<Class, String> managedObjects; // Class for a key

	private Class defaultManagedClass;

	private String defaultManagedRecordType;

	// sqlmap 공유값
	private final static String COUNT = "count";

	private final static String HOW_MANY_ISSUED = "howManyIssued";

	private final static String HOW_MANY_DELETED = "howManyDeleted";

	private final static String METHOD = "method";

	private final static String RECORD_TYPE = "recordType";

	private static final String ISSUE = "issue";



	/////////////////////////////////////////////////////////////////////
	// 1. business operation										   //
	/////////////////////////////////////////////////////////////////////

	/////////////////////////////////////////////////////////////////////
	// 1. 1 checker											   			//
	/////////////////////////////////////////////////////////////////////

	public boolean supports(Class clazz) {

		if(clazz == null)return false;

		return managedObjects.containsKey(clazz);
	}

	public boolean supports(String recordType) {

		if(recordType == null){
			return false;
		}else{
			recordType = recordType.trim();
		}

		return managedObjects.containsValue(recordType);
	}

	/*
	 * 주어진 갱신유형의 지원 여부 확인
	 *
	 * @param updateType
	 *            1 - 추가에 따른 갱신, -1 - 삭제에 따른 갱신
	 * @return 지원 여부
	 */
	private boolean supports(int updateType) {
		if (updateType == ManagedObjectDao.INSERT
				|| updateType == ManagedObjectDao.DELETE) {
			return true;
		} else {
			return false;
		}
	}

	/////////////////////////////////////////////////////////////////////
	// 1. 2 operation for the default managed object				   //
	/////////////////////////////////////////////////////////////////////

	public Integer issue() {

		String recordType = getDefaultManagedObject();

		if( recordType == null){
			return -1;
		}else{
			return numberOf(ISSUE, recordType);
		}

	}

	public Integer howManyDeleted() {

		String recordType = getDefaultManagedObject();

		if( recordType == null){
			return -1;
		}else{
			return  numberOf(HOW_MANY_DELETED, recordType);
		}

	}
	public Integer howManyIssued() {

		String recordType = getDefaultManagedObject();

		if( recordType == null){
			return -1;
		}else{
			return  numberOf(HOW_MANY_ISSUED, recordType);
		}


	}

	public Integer count() {

		String recordType = getDefaultManagedObject();

		if( recordType == null){
			return -1;
		}else{
			return  numberOf(COUNT, recordType);
		}

	}

	/**
	 * 지원하지 않은 갱신유형이거나 기본 관리대상이 지정되어 있지 않으면
	 * @link{IllegalArgumentException} 발생
	 * @see net.webapp2.common.managedobject.dao.ManagedObjectDao#updateFor(int)
	 * @see IllegalArgumentException
	 */
	public void updateFor(int updateType) {

		if(!supports(updateType)){
			logger.warn("INVALID UPDATE TYPE: " + updateType);
			throw new IllegalArgumentException("INVALID UPDATE TYPE");
		}

		String recordType = getDefaultManagedObject();

		if( recordType == null){
			throw new IllegalArgumentException("NO DEFAULT MANAGED OBJECT DEFINDED");
		}else{
			update(updateType, recordType);
		}

	}

	/*
	 * 기본 관리대상을 레코드 타입으로 반환
	 * @return 기본 관리대상에 대한 레코드 타입, 지정된 기본 관리대상이 없으면 null
	 */
	private String getDefaultManagedObject() {

		if(defaultManagedRecordType != null){
			return defaultManagedRecordType;
		}else if(defaultManagedClass != null){
			return managedObjects.get(defaultManagedClass);
		}else{
			logger.warn("NO DEFAULT MANAGED OBJECT DEFINDED");
			return null;
		}

	}

	/////////////////////////////////////////////////////////////////////
	// 1. 3 operation with a given record type						   //
	/////////////////////////////////////////////////////////////////////

	public Integer issue(String recordType) {

		if (!supports(recordType)) {
			logger.warn("INVALID RECORD TYPE: " + recordType);
			return -1;
		}else{
			return numberOf(ISSUE, recordType);
		}

	}

	public Integer howManyDeleted(String recordType) {

		if (!supports(recordType)) {
			logger.warn("INVALID RECORD TYPE: " + recordType);
			return -1;
		}else{
			return numberOf(HOW_MANY_DELETED, recordType);
		}

	}

	public Integer count(String recordType) {

		if (!supports(recordType)) {
			logger.warn("INVALID RECORD TYPE: " + recordType);
			return -1;
		}else{
			return numberOf(COUNT, recordType);
		}
	}


	public int howManyIssued(String recordType) {

		if (!supports(recordType)) {
			logger.warn("INVALID RECORD TYPE: " + recordType);
			return -1;
		}else{
			return numberOf(HOW_MANY_ISSUED, recordType);
		}
	}

	/**
	 * 지원하지 않은 갱신유형 혹은 레코드 유형을 매개변수로 제공시
	 * @link{IllegalArgumentException} 발생
	 * @see net.webapp2.common.managedobject.dao.ManagedObjectDao#updateFor(int, java.lang.String)
	 * @see IllegalArgumentException
	 */
	public void updateFor(int updateType, String recordType) {

		if(!supports(updateType)){
			logger.warn("INVALID UPDATE TYPE: " + updateType);
			throw new IllegalArgumentException("INVALID UPDATE TYPE");
		}else if(! supports(recordType)){
			logger.warn("INVALID RECORD TYPE: " + recordType);
			throw new IllegalArgumentException("INVALID RECORD TYPE");
		}

		update(updateType, recordType);

	}



	/////////////////////////////////////////////////////////////////////
	// 1. 4 operation with a given class							   //
	/////////////////////////////////////////////////////////////////////

	public Integer issue(Class clazz) {

		if (!supports(clazz)) {
			logger.warn("INVALID CLASS: " + clazz);
			return -1;
		}else{
			return numberOf(ISSUE, managedObjects.get(clazz));
		}

	}

	public Integer howManyDeleted(Class clazz) {

		if (!supports(clazz)) {
			logger.warn("INVALID CLASS: " + clazz);
			return -1;
		}else{
			return numberOf(HOW_MANY_DELETED, managedObjects.get(clazz));
		}

	}

	public Integer count(Class clazz) {

		if (!supports(clazz)) {
			logger.warn("INVALID CLASS: " + clazz);
			return -1;
		}else{
			return numberOf(COUNT, managedObjects.get(clazz));
		}

	}

	public int howManyIssued(Class clazz) {

		if (!supports(clazz)) {
			logger.warn("INVALID CLASS: " + clazz);
			return -1;
		}else{
			return numberOf(HOW_MANY_ISSUED, managedObjects.get(clazz));
		}
	}

	/**
	 * 지원하지 않은 갱신유형 혹은 클래스를 매개변수로 제공시
	 * @link{IllegalArgumentException} 발생
	 * @see net.webapp2.common.managedobject.dao.ManagedObjectDao#updateFor(int, java.lang.Class)
	 * @see IllegalArgumentException
	 */
	public void updateFor(int updateType, Class clazz) {

		if(!supports(updateType)){
			logger.warn("INVALID UPDATE TYPE: " + updateType);
			throw new IllegalArgumentException("INVALID UPDATE TYPE");
		}else if(! supports(clazz)){
			logger.warn("INVALID CLASS: " + clazz);
			throw new IllegalArgumentException("INVALID CLASS");
		}

		update(updateType, managedObjects.get(clazz));

	}


	/////////////////////////////////////////////////////////////////////
	// 1. 5 real worker												   //
	/////////////////////////////////////////////////////////////////////

	/*
	 * count, how many XXX 등의 수치를 구하는 쿼리 공통화
	 * @param method 수치 유형
	 * @param recordType 레코드 유형
	 * @return
	 */
	private Integer numberOf(String method, String recordType) {

		Map<String, String> params = new HashMap<String, String>();
		params.put(METHOD, method);
		params.put(RECORD_TYPE, recordType);

		Integer number = (Integer) getSqlMapClientTemplate().queryForObject(
				"ManagedObject.numberOf", params);

		logger.debug(method + ":" + number + " for " + recordType);

		return number;

	}



	/*
	 * updateFor 오퍼레이션의 실제 연산 수행
	 * @param updateType
	 * @param recordType
	 */
	private void update(int updateType, String recordType) {

		Map<String, Object> params = new HashMap<String, Object>();
		params.put("updateType", updateType);
		params.put("recordType", recordType);

		logger.debug("update managed object data: " +
				"updateType is " + updateType +
				", recordType" + recordType);

		getSqlMapClientTemplate().update("ManagedObject.update", params);

	}


	/////////////////////////////////////////////////////////////////////
	// setters														   //
	/////////////////////////////////////////////////////////////////////

	public void setDefaultManagedClass(Class defaultManagedClass) {
		this.defaultManagedClass = defaultManagedClass;
	}

	public void setDefaultManagedRecordType(String defaultManagedRecordType) {
		this.defaultManagedRecordType = defaultManagedRecordType;
	}

	public void setManagedObjects(Map<Class, String> managedObjects) {
		this.managedObjects = managedObjects;
	}

}
