package net.webapp2.util.date;

import java.util.Date;
import java.util.GregorianCalendar;

/**
 * 날짜처리에 관련한 JDK의 번거로운 API를 단순화
 *
 * @author younghoe
 *
 */
public class DateUtils {

	private static final int MONTH_OFFSET = 1;

	/**
	 * 연,월,일 정수를 받아 Date 객체 생성. 일의 경우 각 월의 마지막 날을 초과하는 숫자를 입력하는 경우 익월로 계산된다.
	 *
	 * @param year
	 *            연(1 이상의 값을 넣어야 함)
	 * @param month
	 *            월(1~12)
	 * @param dayOfMonth
	 *            일(1~31)
	 * @return Date 객체
	 */
	public static final Date getDate(int year, int month, int dayOfMonth) {

		if (year < 1)
			throw new IllegalArgumentException("0년 이하의 값은 입력하실 수 없습니다.");
		else if (month < 1 || month > 12)
			throw new IllegalArgumentException("1 ~ 12월 사이의 값만 입력하실 수 있습니다.");
		else if (dayOfMonth < 1 || dayOfMonth > 31)
			throw new IllegalArgumentException("1 ~ 31일 사이의 값만 입력하실 수 있습니다.");

		return new GregorianCalendar(year, month - MONTH_OFFSET,
				dayOfMonth).getTime();

	}

}
