package net.webapp2.util.ibatis;

import java.sql.SQLException;
import java.sql.Types;

import com.ibatis.sqlmap.client.extensions.ParameterSetter;
import com.ibatis.sqlmap.client.extensions.ResultGetter;
import com.ibatis.sqlmap.client.extensions.TypeHandlerCallback;

/**
 * iBatis에서 enum 타입의 값을 처리하기 위한 iBatis Template 메소드 구현 클래스
 * @author younghoe
 *
 * @param <E> 처리 대상이 되는 enum 타입
 */
public abstract class EnumTypeHandler<E extends Enum> implements
		TypeHandlerCallback {

	/* (non-Javadoc)
	 * @see com.ibatis.sqlmap.client.extensions.TypeHandlerCallback#setParameter(com.ibatis.sqlmap.client.extensions.ParameterSetter, java.lang.Object)
	 */
	public void setParameter(ParameterSetter setter, Object parameter)
			throws SQLException {

		if (parameter == null) {
			setter.setNull(Types.VARCHAR);
		} else {
			E enumValue = (E) parameter;
			setter.setString(enumValue.toString());
		}

	}

	/* (non-Javadoc)
	 * @see com.ibatis.sqlmap.client.extensions.TypeHandlerCallback#getResult(com.ibatis.sqlmap.client.extensions.ResultGetter)
	 */
	public E getResult(ResultGetter getter) throws SQLException {

		if (getter.wasNull())
			return null;

		String value = getter.getString();

		for (E enumValue : getEnums()) {
			if (enumValue.toString().equals(value)) {
				return enumValue;
			}
		}

		throw new UnsupportedOperationException("enum에 정의되어 있지 않은 값이 반환되었습니다.");

	}

	/**
	 * enum에 포함된 값들을 모두 반환. 하위 클래스에서 실질적으로 구현해야 할 메소드.
	 * @return enum의 모든 값을 담은 배열
	 */
	protected abstract E[] getEnums();

	/* (non-Javadoc)
	 * @see com.ibatis.sqlmap.client.extensions.TypeHandlerCallback#valueOf(java.lang.String)
	 */
	public E valueOf(String value) {

		if (value == null)
			return null;

		for (E enumValue : getEnums()) {
			if (enumValue.toString().equals(value)) {
				return enumValue;
			}
		}

		return null;

	}

}
