package net.webapp2.util.test;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Properties;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.beans.factory.xml.XmlBeanFactory;
import org.springframework.core.io.FileSystemResource;

/**
 * 테스트 과정에서 Spring 컨텍스트 객체 역할 수행
 * @author younghoe
 *
 */
public class SpringMock {

	protected final Log logger = LogFactory.getLog(getClass());

	private static final String APPLICATION_CONTEXT_FILE_PATH_PROPERTY = "application.context.file.path";

	private static final String DAO_CONTEXT_FILE_PATH_PROPERTY = "dao.context.file.path";

	public static final String DEFAULT_CONTEXT_PROPERTIES_FILE_PATH = "conf/context.properties";

	/**
	 * context.properties 파일의 위치. 디폴트는
	 * DEFAULT_CONTEXT_CONFIG_PROPERTIES_FILE_PATH
	 */
	private String contextPropertiesFilePath = DEFAULT_CONTEXT_PROPERTIES_FILE_PATH;


	public BeanFactory loadContext() throws FileNotFoundException, IOException, PropertyDefinitionNotFoundException {

		// load config
		logger.info(contextPropertiesFilePath + " 파일의 " + "설정 사항을 읽어옵니다.");

		Properties props = new Properties();
		props.load(new FileInputStream(contextPropertiesFilePath));

		String applicationContextFilePath = props
				.getProperty(APPLICATION_CONTEXT_FILE_PATH_PROPERTY);
		String daoContextFilePath = props
				.getProperty(DAO_CONTEXT_FILE_PATH_PROPERTY);

		if (logger.isDebugEnabled()) {
			logger.debug(APPLICATION_CONTEXT_FILE_PATH_PROPERTY + "="
					+ applicationContextFilePath);
			logger.debug(DAO_CONTEXT_FILE_PATH_PROPERTY + "="
					+ daoContextFilePath);
		}

		// load daoContext.xml
		if (daoContextFilePath == null || daoContextFilePath.equals("")) {
			throw new PropertyDefinitionNotFoundException(
					DAO_CONTEXT_FILE_PATH_PROPERTY + " 프로퍼티 설정을 찾을 수 없습니다.");
		}

		XmlBeanFactory parent = new XmlBeanFactory(new FileSystemResource(
				daoContextFilePath));

		// load applicationContext.xml
		if (applicationContextFilePath == null
				|| applicationContextFilePath.equals("")) {
			throw new PropertyDefinitionNotFoundException(
					APPLICATION_CONTEXT_FILE_PATH_PROPERTY
							+ " 프로퍼티 설정을 찾을 수 없습니다.");
		}

		XmlBeanFactory appContext = new XmlBeanFactory(new FileSystemResource(
				applicationContextFilePath), parent);

		return (BeanFactory) appContext;

	}


	/**
	 * conf/context.properties로 설정된 컨텍스트 프로퍼티 파일 경로를 변경함
	 * @param contextPropertiesFilePath 컨텍스트 프로퍼티 파일 경로
	 */
	public void setContextPropertiesFilePath(String contextPropertiesFilePath) {

		this.contextPropertiesFilePath = contextPropertiesFilePath;

	}

}
