package net.webapp2.common.managedobject.dao.ibatis;

import java.util.HashMap;
import java.util.Map;

import javax.sql.DataSource;

import net.webapp2.common.managedobject.dao.ManagedObjectDao;
import net.webapp2.util.jdbc.SimpleSqlExecutor;
import net.webapp2.util.test.TestBase;

import org.testng.annotations.Configuration;
import org.testng.annotations.ExpectedExceptions;
import org.testng.annotations.Test;

public class ManagedObjectDaoSupportTest extends TestBase{

	/**
	 * (Object.class, "object"), (String.class, "string" ,
	 * (Integer.class, "integer")를 지원하면 기본 관리 대상은 object
	 */
	private ManagedObjectDao managedObjectDao;

	@Test(groups="function")
	public void 관리대상의_타입_및_레코드_타입_확인(){

		isTrue(managedObjectDao.supports(Object.class));
		isTrue(managedObjectDao.supports(String.class));
		isTrue(managedObjectDao.supports(Integer.class));
		isFalse(managedObjectDao.supports(ManagedObjectDaoSupportTest.class));

		isTrue(managedObjectDao.supports("object"));
		isTrue(managedObjectDao.supports("string"));
		isTrue(managedObjectDao.supports("integer"));
		isFalse(managedObjectDao.supports("webapp2.net"));

	}

	@Test(groups="function")
	public void 관리번호_발행(){

		Integer no = managedObjectDao.issue();
		equals(1, no);

	}

	@Test(groups="function")
	public void 관리번호_발행시_특정_레코드_타입_지정(){

		Integer no = managedObjectDao.issue("string");
		equals(1, no);

		no = managedObjectDao.issue("object");
		equals(1, no);

	}

	@Test(groups="function")
	public void 관리번호_발행시_특정_클래스_지정(){

		Integer no = managedObjectDao.issue(Object.class);
		equals(1, no);

		no = managedObjectDao.issue(String.class);
		equals(1, no);

	}

	@Test(groups="function")
	public void 관리데이터갱신_데이터추가시(){

		// before update
		Integer no = managedObjectDao.issue(Object.class);
		equals(1, no);

		// 1. do update with type
		managedObjectDao.updateFor(ManagedObjectDao.INSERT, Object.class);

		// after update
		no = managedObjectDao.issue(Object.class);
		equals(2, no);

		// 2. do update with recore type
		managedObjectDao.updateFor(ManagedObjectDao.INSERT, "object");

		no = managedObjectDao.issue(Object.class);
		equals(3, no);

		// 3. do update in default
		managedObjectDao.updateFor(ManagedObjectDao.INSERT);

		no = managedObjectDao.issue();
		equals(4, no);

		// 4. do update another record type
		managedObjectDao.updateFor(ManagedObjectDao.INSERT, "string");

		no = managedObjectDao.issue("string");
		equals(2, no);

	}

	@Test(groups="function")
	public void 관리데이터갱신_데이터삭제시(){

		// before update
		Integer no = managedObjectDao.howManyDeleted(Object.class);
		equals(0, no);

		// 1. do update with type
		managedObjectDao.updateFor(ManagedObjectDao.DELETE, Object.class);

		// after update
		no = managedObjectDao.howManyDeleted(Object.class);
		equals(1, no);

		// 2. do update with recore type
		managedObjectDao.updateFor(ManagedObjectDao.DELETE, "object");

		no = managedObjectDao.howManyDeleted(Object.class);
		equals(2, no);

		// 3. do update in default
		managedObjectDao.updateFor(ManagedObjectDao.DELETE);

		no = managedObjectDao.howManyDeleted();
		equals(3, no);

		// 4. do update another record type
		managedObjectDao.updateFor(ManagedObjectDao.DELETE, "string");

		no = managedObjectDao.howManyDeleted("string");
		equals(1, no);

	}

	@Test(groups="function")
	public void 발행_삭제_갱신(){

		int result_count = managedObjectDao.count();

		// initial state
		equals(0, result_count);

		int issue_count = 10;
		for(int i = 1; i <= issue_count; i++){
			equals(i, managedObjectDao.issue());
			managedObjectDao.updateFor(ManagedObjectDao.INSERT);
		}

		// after 10 issert
		result_count = managedObjectDao.count();
		equals(issue_count, result_count);

		int del_count = 5;
		for(int i = 1; i <= del_count; i++){
			managedObjectDao.updateFor(ManagedObjectDao.DELETE);
		}

		// after 3 del
		result_count = managedObjectDao.howManyDeleted();
		equals(del_count, result_count);

		result_count = managedObjectDao.howManyIssued();
		equals(issue_count, result_count);

		result_count = managedObjectDao.count();
		equals(issue_count - del_count, result_count);

		// multi 지원(count, howManyIssued)
		result_count = managedObjectDao.count(Object.class);
		equals(issue_count - del_count, result_count);

		result_count = managedObjectDao.count("string");
		equals(0, result_count);

		result_count = managedObjectDao.howManyIssued(String.class);
		equals(0, result_count);

		result_count = managedObjectDao.howManyIssued("object");
		equals(issue_count, result_count);


	}

	@Test(groups="exception")
	public void supports_NULL_체크(){

		boolean bool = managedObjectDao.supports((Class)null);
		isFalse(bool);

		bool = managedObjectDao.supports((String)null);
		isFalse(bool);

	}

	@Test(groups="exception")
	public void issue_클래스나_레코드타입으로_NULL(){

		equals(-1, managedObjectDao.issue((Class)null));
		equals(-1, managedObjectDao.issue((String)null));

	}


	@Test(groups="exception")
	@ExpectedExceptions(IllegalArgumentException.class)
	public void updateFor_잘못된_갱신유형(){

		managedObjectDao.updateFor(555);

	}

	@Test(groups="exception")
	@ExpectedExceptions(IllegalArgumentException.class)
	public void updateFor_잘못된_갱신유형과_적절한_타입(){

		managedObjectDao.updateFor(555, Object.class);

	}

	@Test(groups="exception")
	@ExpectedExceptions(IllegalArgumentException.class)
	public void updateFor_잘못된_갱신유형과_적절한_레코드타입(){
		System.out.println("#### " + managedObjectDao);
		managedObjectDao.updateFor(555, "string");

	}

	@Test(groups="exception")
	@ExpectedExceptions(IllegalArgumentException.class)
	public void updateFor_잘못된_레코드타입(){
		System.out.println("#### " + managedObjectDao);
		managedObjectDao.updateFor(ManagedObjectDao.INSERT, "younghoe");

	}

	@Test(groups="exception")
	@ExpectedExceptions(IllegalArgumentException.class)
	public void updateFor_잘못된_타입(){
		System.out.println("#### " + managedObjectDao);
		managedObjectDao.updateFor(ManagedObjectDao.DELETE, ManagedObjectDao.class);

	}

	@Test(groups="exception")
	public void howManyDeleted_클래스나_레코드타입으로_NULL(){

		equals(-1, managedObjectDao.howManyDeleted((Class)null));
		equals(-1, managedObjectDao.howManyDeleted((String)null));

	}

	@Test(groups="exception")
	public void count_클래스나_레코드타입으로_NULL(){

		equals(-1, managedObjectDao.count((Class)null));
		equals(-1, managedObjectDao.count((String)null));

	}


	@Test(groups="exception")
	public void howManyIssued_클래스나_레코드타입으로_NULL(){

		equals(-1, managedObjectDao.howManyIssued((Class)null));
		equals(-1, managedObjectDao.howManyIssued((String)null));

	}


	/////////////////////////////////////////////////////////////////////
	// non-test method 												   //
	/////////////////////////////////////////////////////////////////////

	@SuppressWarnings("unused")
	@Configuration(beforeTestMethod=true, groups="always")
	private void 데이터베이스_초기화() {

		SimpleSqlExecutor executor = new SimpleSqlExecutor();
		DataSource dataSource = (DataSource) getBean("dataSource");
		executor.setDataSource(dataSource);

		String statement = "DROP TABLE ManagedObject" +
				" IF EXISTS";
		executor.execute(statement);

		statement = "CREATE TABLE ManagedObject(" +
						"type VARCHAR(10) PRIMARY KEY," +
						"issue INTEGER NOT NULL," +
						"del INTEGER NOT NULL);";
		executor.execute(statement);

		statement = "INSERT INTO ManagedObject VALUES('object', 0, 0);";
		executor.execute(statement);

		statement = "INSERT INTO ManagedObject VALUES('string', 0, 0);";
		executor.execute(statement);

	}


	@SuppressWarnings("unused")
	@Configuration(beforeTestMethod=true, groups="always")
	private void 기본설정(){

		ManagedObjectDaoSupport managedObjectDaoSupport = (ManagedObjectDaoSupport) getBean("managedObjectDao");

		Map<Class, String> managedObjects = new HashMap<Class, String>();
		managedObjects.put(Object.class, "object");
		managedObjects.put(String.class, "string");
		managedObjects.put(Integer.class, "integer");
		managedObjectDaoSupport.setManagedObjects(managedObjects);

		managedObjectDaoSupport.setDefaultManagedClass(Object.class);
		managedObjectDaoSupport.setDefaultManagedRecordType("object");

		this.managedObjectDao = managedObjectDaoSupport;

	}

}
