package net.webapp2.util.ibatis;

import java.io.FileNotFoundException;
import java.io.IOException;

import net.webapp2.util.ibatis.misc.District;
import net.webapp2.util.ibatis.misc.DistrictTypeHandler;
import net.webapp2.util.ibatis.misc.JustForTest;
import net.webapp2.util.ibatis.misc.JustForTestDao;
import net.webapp2.util.jdbc.SimpleSqlExecutor;
import net.webapp2.util.test.PropertyDefinitionNotFoundException;
import net.webapp2.util.test.TestBase;

import org.springframework.jdbc.UncategorizedSQLException;
import org.testng.annotations.ExpectedExceptions;
import org.testng.annotations.Parameters;
import org.testng.annotations.Test;

import com.ibatis.sqlmap.client.extensions.TypeHandlerCallback;

public class EnumTypeHandlerTest extends TestBase {

	@Test(groups = "function")
	public void TypeHandler인터페이스준수여부() {

		DistrictTypeHandler handler = new DistrictTypeHandler();
		isTrue(handler instanceof TypeHandlerCallback);

	}

	@Test(groups = "function")
	public void SqlMap에서_Param_처리() {

		테스트_테이블_새로_생성();

		JustForTestDao testDao = (JustForTestDao) getBean("testDao");
		JustForTest test = new JustForTest();
		test.setId("1");
		test.setDistrict(District.seoul);

		testDao.insert(test);

	}

	@Test(groups = "function")
	public void SqlMap에서_결과받기() {

		테스트_데이터_입력();

		JustForTestDao testDao = (JustForTestDao) getBean("testDao");
		JustForTest test = testDao.findById("1");

		equals("1", test.getId());
		equals(District.seoul, test.getDistrict());

	}

	@Test(groups = "exception")
	@Parameters("EnumTypeHandler가_등록되어_있지_않을때")
	@ExpectedExceptions(UncategorizedSQLException.class)
	public void EnumTypeHandler가_등록되어_있지_않을때(String fileName)
			throws FileNotFoundException, IOException,
			PropertyDefinitionNotFoundException {

		테스트_테이블_새로_생성();
		테스트_데이터_입력();

		TestBase test = new Temp(fileName);
		JustForTestDao testDao = (JustForTestDao) test.getBean("testDao");
		testDao.findById("1");

	}

	// ///////////////////////////////////////////////////////////////////
	// non-test method //
	// ///////////////////////////////////////////////////////////////////

	private class Temp extends TestBase {

		public Temp(String fileName) throws FileNotFoundException, IOException,
				PropertyDefinitionNotFoundException {
			super(fileName);
		}

		public Temp() {
			super();
		}

	}

	private void 테스트_테이블_새로_생성() {

		SimpleSqlExecutor executor = getSqlExecutor();

		String dropTableStatement = "DROP TABLE Test" + " IF EXISTS";
		executor.execute(dropTableStatement);

		String createTableStatement = "CREATE TABLE Test("
				+ " id VARCHAR(10), district VARCHAR(10))";
		executor.execute(createTableStatement);

	}

	private void 테스트_데이터_입력() {

		테스트_테이블_새로_생성();

		SimpleSqlExecutor executor = getSqlExecutor();

		String insertStatement = "INSERT INTO Test VALUES('1', 'seoul')";
		executor.execute(insertStatement);

	}

}
