package net.webapp2.board.web;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.webapp2.board.domain.Article;
import net.webapp2.board.domain.Writing;
import net.webapp2.board.service.ArticleService;
import net.webapp2.util.RequestUtil;
import net.webapp2.util.StringUtil;

import org.springframework.validation.BindException;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.SimpleFormController;
import org.springframework.web.servlet.view.RedirectView;

public class FormController extends SimpleFormController {

	private ArticleService	articleService;

	// 특정상황에 맞춰서 form에 value들을 셋팅한다.
	protected Object formBackingObject(HttpServletRequest request) throws Exception {
		if(request.getParameter("id")==null) {
			return new Article();
		} else {
			int id = RequestUtil.getIntParam(request, "id", 1);
			Writing article = articleService.view(id);
			// 실제 글 내용중의 <br/>을 공백으로 치환.
			String content = StringUtil.replaceAll(article.getContents(), "<br/>", "");
			article.setContents(content);
			return article;
		}
	}

	// submit이 일어 났을때 호출 되는 method
	protected ModelAndView onSubmit
	(HttpServletRequest request, HttpServletResponse response, Object command, BindException errors)
	throws Exception {

		ModelAndView successView = null;
		Writing article = (Writing) command;
		// 글 내용중의 개행문자를 <br/>로 치환
		String content = StringUtil.replaceAll(article.getContents(), "\n", "<br/>");
		article.setContents(content);

		if(article.getId() == null) {
			// article을 등록한다.
			articleService.insert(article);
			// application-context에 설정된 successview로 ModelAndView를 생성하고 model에 값을 설정한다.
			successView = new ModelAndView(new RedirectView(getSuccessView()));
		} else {
			// article을 수정한다.
			// to-do : update 메소드의 결과값에 따라 성공 실패 구분하여 처리? -> 트랜잭션필요.
			articleService.update(article);
			successView = new ModelAndView(new RedirectView("view.html"));
			successView.addObject("id", article.getId());
		}

		return successView;

	}

	public void setArticleService(ArticleService articleService) {
		this.articleService = articleService;
	}
}
